@@ -11,8 +11,8 @@ if not modules then modules = { } end modules ['mtx-epub'] = {
 -- really an id but has some special property). Then there is this ncx suffix
 -- thing. Somehow it give the impression of a reversed engineered application
 -- format so it will probably take a few cycles to let it become a real
--- clean standard. Thanks to Adam Reviczky for helping to figure out all these
--- puzzling details.
+-- clean standard. Thanks to Adam Reviczky, Luigi Scarso and Andy Thomas for
+-- helping to figure out all the puzzling details.
 
 -- This is preliminary code. At some point we will deal with images as well but
 -- first we need a decent strategy to export them. More information will be
@@ -22,16 +22,34 @@ local format, gsub = string.format, string.gsub
 local concat = table.concat
 
 local helpinfo = [[
---make                create epub zip file
-
-example:
-
-mtxrun --script epub --make mydocument
+<?xml version="1.0"?>
+<application>
+ <metadata>
+  <entry name="name">mtx-epub</entry>
+  <entry name="detail">ConTeXt EPUB Helpers</entry>
+  <entry name="version">0.12</entry>
+ </metadata>
+ <flags>
+  <category name="basic">
+   <subcategory>
+    <flag name="make"><short>create epub zip file</short></flag>
+   </subcategory>
+  </category>
+ </flags>
+ <examples>
+  <category>
+   <title>Example</title>
+   <subcategory>
+    <example><command>mtxrun --script epub --make mydocument</command></example>
+   </subcategory>
+  </category>
+ </examples>
+</application>
 ]]
 
 local application = logs.application {
     name     = "mtx-epub",
-    banner   = "ConTeXt EPUB Helpers 0.11",
+    banner   = "ConTeXt EPUB Helpers 0.12",
     helpinfo = helpinfo,
 }
 
@@ -43,26 +61,27 @@ scripts.epub = scripts.epub or { }
 local mimetype = "application/epub+zip"
 
 local container = [[
-<?xml version="1.0" encoding="UTF-8" ?>
+<?xml version="1.0" encoding="UTF-8"?>
 
 <container version="1.0" xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
     <rootfiles>
-        <rootfile full-path="OPS/%s" media-type="application/oebps-package+xml"/>
+        <rootfile full-path="OEBPS/%s" media-type="application/oebps-package+xml"/>
     </rootfiles>
 </container>
 ]]
 
 local package = [[
-<?xml version="1.0"?>
+<?xml version="1.0" encoding="UTF-8"?>
 
 <package version="2.0" xmlns="http://www.idpf.org/2007/opf" unique-identifier="%s">
 
     <metadata xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:opf="http://www.idpf.org/2007/opf">
-        <dc:title>My Title</dc:title>
-        <dc:language>en</dc:language>
-        <dc:identifier id="%s" >urn:uuid:%s</dc:identifier>
-        <dc:creator opf:file-as="Self, My" opf:role="aut">MySelf</dc:creator>
+        <dc:title>%s</dc:title>
+        <dc:language>%s</dc:language>
+        <dc:identifier id="%s" opf:scheme="UUID">urn:uuid:%s</dc:identifier>
+        <dc:creator>%s</dc:creator>
         <dc:date>%s</dc:date>
+        <meta name="cover" content="%s" />
     </metadata>
 
     <manifest>
@@ -70,13 +89,14 @@ local package = [[
     </manifest>
 
     <spine toc="ncx">
+        <itemref idref="cover-xhtml" />
         <itemref idref="%s" />
     </spine>
 
 </package>
 ]]
 
-local item = [[        <item id='%s' href='%s' media-type='%s'/>]]
+local item = [[        <item id="%s" href="%s" media-type="%s"/>]]
 
 local toc = [[
 <?xml version="1.0"?>
@@ -108,6 +128,23 @@ local toc = [[
 </ncx>
 ]]
 
+local coverxhtml = [[
+<?xml version="1.0" encoding="UTF-8"?>
+
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
+
+<html xmlns="http://www.w3.org/1999/xhtml">
+    <head>
+        <title>cover.xhtml</title>
+    </head>
+    <body>
+        <div>
+            <img src="%s" alt="The cover image" style="max-width: 100%%;" />
+        </div>
+    </body>
+</html>
+]]
+
 -- We need to figure out what is permitted. Numbers only seem to give
 -- problems is some applications as do names with dashes. Also the
 -- optional toc is supposed to be there and although id's are by
@@ -117,7 +154,7 @@ local toc = [[
 
 local function dumbid(filename)
  -- return (string.gsub(os.uuid(),"%-%","")) -- to be tested
-    return file.nameonly(filename) .. "-" .. file.extname(filename)
+    return file.nameonly(filename) .. "-" .. file.suffix(filename)
 end
 
 local mimetypes = {
@@ -128,6 +165,7 @@ local mimetypes = {
     png     = "image/png",
     jpg     = "image/jpeg",
     ncx     = "application/x-dtbncx+xml",
+    gif     = "image/gif",
  -- default = "text/plain",
 }
 
@@ -194,9 +232,21 @@ function scripts.epub.make()
         local files      = specification.files      or { file.addsuffix(filename,"xhtml") }
         local images     = specification.images     or { }
         local root       = specification.root       or files[1]
+        local language   = specification.language   or "en"
+        local creator    = specification.author     or "My Self"
+        local title      = specification.title      or "My Title"
+        local firstpage  = specification.firstpage  or ""
+        local lastpage   = specification.lastpage   or ""
 
      -- identifier = gsub(identifier,"[^a-zA-z0-9]","")
 
+        if firstpage ~= "" then
+            images[firstpage] = firstpage
+        end
+        if lastpage ~= "" then
+            images[lastpage] = lastpage
+        end
+
         identifier = "BookId" -- weird requirement
 
         local epubname   = name
@@ -204,11 +254,12 @@ function scripts.epub.make()
         local epubfile   = file.replacesuffix(name,"epub")
         local epubroot   = file.replacesuffix(name,"opf")
         local epubtoc    = "toc.ncx"
+        local epubcover  = "cover.xhtml"
 
         application.report("creating paths in tree %s",epubpath)
         lfs.mkdir(epubpath)
         lfs.mkdir(file.join(epubpath,"META-INF"))
-        lfs.mkdir(file.join(epubpath,"OPS"))
+        lfs.mkdir(file.join(epubpath,"OEBPS"))
 
         local used = { }
 
@@ -217,13 +268,14 @@ function scripts.epub.make()
             local mime = mimetypes[suffix]
             if mime then
                 local idmaker = idmakers[suffix] or idmakers.default
-                local target = file.join(epubpath,"OPS",filename)
+                local target = file.join(epubpath,"OEBPS",filename)
                 file.copy(filename,target)
                 application.report("copying %s to %s",filename,target)
                 used[#used+1] = format(item,idmaker(filename),filename,mime)
             end
         end
 
+        copyone("cover.xhtml")
         copyone("toc.ncx")
 
         local function copythem(files)
@@ -241,7 +293,7 @@ function scripts.epub.make()
 
         for k, v in table.sortedpairs(images) do
             theimages[#theimages+1] = k
-            if not lfs.isfile(k) and file.extname(k) == "svg" and file.extname(v) == "pdf" then
+            if not lfs.isfile(k) and file.suffix(k) == "svg" and file.suffix(v) == "pdf" then
                 local command = format("inkscape --export-plain-svg=%s %s",k,v)
                 application.report("running command '%s'\n\n",command)
                 os.execute(command)
@@ -250,33 +302,52 @@ function scripts.epub.make()
 
         copythem(theimages)
 
-        local idmaker = idmakers[file.extname(root)] or idmakers.default
+        local idmaker = idmakers[file.suffix(root)] or idmakers.default
 
-        container = format(container,epubroot)
-        package   = format(package,identifier,identifier,os.uuid(),os.date("!%Y-%m-%dT%H:%M:%SZ"),concat(used,"\n"),idmaker(root))
-        toc       = format(toc,identifier,"title",root)
+        container = format(container,
+            epubroot
+        )
+        package = format(package,
+            identifier,
+            title,
+            language,
+            identifier,
+            os.uuid(),
+            creator,
+            os.date("!%Y-%m-%dT%H:%M:%SZ"),
+            idmaker(firstpage),
+            concat(used,"\n"),
+            idmaker(root)
+        )
+        toc = format(toc,
+            identifier,
+            title,
+            root
+        )
+        coverxhtml = format(coverxhtml,
+            firstpage
+        )
 
         io.savedata(file.join(epubpath,"mimetype"),mimetype)
         io.savedata(file.join(epubpath,"META-INF","container.xml"),container)
-        io.savedata(file.join(epubpath,"OPS",epubroot),package)
-        io.savedata(file.join(epubpath,"OPS",epubtoc),toc)
+        io.savedata(file.join(epubpath,"OEBPS",epubroot),package)
+        io.savedata(file.join(epubpath,"OEBPS",epubtoc),toc)
+        io.savedata(file.join(epubpath,"OEBPS",epubcover),coverxhtml)
 
         application.report("creating archive\n\n")
 
-        local done = false
-        local list = { }
-
         lfs.chdir(epubpath)
         os.remove(epubfile)
 
+        local done = false
+
         for i=1,#zippers do
             local zipper = zippers[i]
             if os.execute(format(zipper.uncompressed,epubfile,"mimetype")) then
                 os.execute(format(zipper.compressed,epubfile,"META-INF"))
-                os.execute(format(zipper.compressed,epubfile,"OPS"))
+                os.execute(format(zipper.compressed,epubfile,"OEBPS"))
                 done = zipper.name
-            else
-                list[#list+1] = zipper.name
+                break
             end
         end
 
@@ -285,6 +356,10 @@ function scripts.epub.make()
         if done then
             application.report("epub archive made using %s: %s",done,file.join(epubpath,epubfile))
         else
+            local list = { }
+            for i=1,#zippers do
+                list[#list+1] = zipper.name
+            end
             application.report("no epub archive made, install one of: %s",concat(list," "))
         end
 
@@ -296,6 +371,8 @@ end
 
 if environment.argument("make") then
     scripts.epub.make()
+elseif environment.argument("exporthelp") then
+    application.export(environment.argument("exporthelp"),environment.files[1])
 else
     application.help()
 end
