@@ -10,16 +10,16 @@ if not modules then modules = { } end modules ['buff-ver'] = {
 -- supposed to use different names for their own variants.
 --
 -- todo: skip=auto
+--
+-- todo: update to match context scite lexing
 
-local type, next, rawset, rawget, setmetatable, getmetatable = type, next, rawset, rawget, setmetatable, getmetatable
+local type, next, rawset, rawget, setmetatable, getmetatable, tonumber = type, next, rawset, rawget, setmetatable, getmetatable, tonumber
 local format, lower, upper,match, find, sub = string.format, string.lower, string.upper, string.match, string.find, string.sub
 local splitlines = string.splitlines
 local concat = table.concat
 local C, P, R, S, V, Carg, Cc, Cs = lpeg.C, lpeg.P, lpeg.R, lpeg.S, lpeg.V, lpeg.Carg, lpeg.Cc, lpeg.Cs
 local patterns, lpegmatch, is_lpeg = lpeg.patterns, lpeg.match, lpeg.is_lpeg
 
-local context, commands = context, commands
-
 local trace_visualize      = false  trackers.register("buffers.visualize", function(v) trace_visualize = v end)
 local report_visualizers   = logs.reporter("buffers","visualizers")
 
@@ -29,6 +29,9 @@ visualizers                = visualizers or { }
 local specifications       = allocate()
 visualizers.specifications = specifications
 
+local context              = context
+local commands             = commands
+
 local tabtospace           = utilities.strings.tabtospace
 local variables            = interfaces.variables
 local settings_to_array    = utilities.parsers.settings_to_array
@@ -38,72 +41,75 @@ local addsuffix            = file.addsuffix
 
 local v_auto               = variables.auto
 local v_yes                = variables.yes
+local v_last               = variables.last
+local v_all                = variables.all
 
 -- beware, all macros have an argument:
 
-local doinlineverbatimnewline    = context.doinlineverbatimnewline
-local doinlineverbatimbeginline  = context.doinlineverbatimbeginline
-local doinlineverbatimemptyline  = context.doinlineverbatimemptyline
-local doinlineverbatimstart      = context.doinlineverbatimstart
-local doinlineverbatimstop       = context.doinlineverbatimstop
+local ctx_inlineverbatimnewline     = context.doinlineverbatimnewline
+local ctx_inlineverbatimbeginline   = context.doinlineverbatimbeginline
+local ctx_inlineverbatimemptyline   = context.doinlineverbatimemptyline
+local ctx_inlineverbatimstart       = context.doinlineverbatimstart
+local ctx_inlineverbatimstop        = context.doinlineverbatimstop
 
-local dodisplayverbatimnewline   = context.dodisplayverbatimnewline
-local dodisplayverbatimbeginline = context.dodisplayverbatimbeginline
-local dodisplayverbatimemptyline = context.dodisplayverbatimemptyline
-local dodisplayverbatimstart     = context.dodisplayverbatimstart
-local dodisplayverbatimstop      = context.dodisplayverbatimstop
+local ctx_displayverbatiminitialize = context.dodisplayverbatiminitialize -- the number of arguments might change over time
+local ctx_displayverbatimnewline    = context.dodisplayverbatimnewline
+local ctx_displayverbatimbeginline  = context.dodisplayverbatimbeginline
+local ctx_displayverbatimemptyline  = context.dodisplayverbatimemptyline
+local ctx_displayverbatimstart      = context.dodisplayverbatimstart
+local ctx_displayverbatimstop       = context.dodisplayverbatimstop
 
-local verbatim                   = context.verbatim
-local doverbatimspace            = context.doverbatimspace
+local ctx_verbatim                  = context.verbatim
+local ctx_verbatimspace             = context.doverbatimspace
 
 local CargOne = Carg(1)
 
 local function f_emptyline(s,settings)
     if settings and settings.nature == "inline" then
-        doinlineverbatimemptyline()
+        ctx_inlineverbatimemptyline()
     else
-        dodisplayverbatimemptyline()
+        ctx_displayverbatimemptyline()
     end
 end
 
 local function f_beginline(s,settings)
     if settings and settings.nature == "inline" then
-        doinlineverbatimbeginline()
+        ctx_inlineverbatimbeginline()
     else
-        dodisplayverbatimbeginline()
+        ctx_displayverbatimbeginline()
     end
 end
 
 local function f_newline(s,settings)
     if settings and settings.nature == "inline" then
-        doinlineverbatimnewline()
+        ctx_inlineverbatimnewline()
     else
-        dodisplayverbatimnewline()
+        ctx_displayverbatimnewline()
     end
 end
 
 local function f_start(s,settings)
     if settings and settings.nature == "inline" then
-        doinlineverbatimstart()
+        ctx_inlineverbatimstart()
     else
-        dodisplayverbatimstart()
+        ctx_displayverbatimstart()
     end
 end
 
 local function f_stop(s,settings)
     if settings and settings.nature == "inline" then
-        doinlineverbatimstop()
+        ctx_inlineverbatimstop()
     else
-        dodisplayverbatimstop()
+        ctx_displayverbatimstop()
     end
 end
 
 local function f_default(s) -- (s,settings)
-    verbatim(s)
+    ctx_verbatim(s)
 end
 
 local function f_space() -- (s,settings)
-    doverbatimspace()
+    ctx_verbatimspace()
 end
 
 local function f_signal() -- (s,settings)
@@ -194,7 +200,7 @@ local function getvisualizer(method,nature)
     end
 end
 
-local fallback = context.verbatim
+local ctx_fallback = ctx_verbatim
 
 local function makepattern(visualizer,replacement,pattern)
     if not pattern then
@@ -202,9 +208,9 @@ local function makepattern(visualizer,replacement,pattern)
         return patterns.alwaystrue
     else
         if type(visualizer) == "table" and type(replacement) == "string" then
-            replacement = visualizer[replacement] or fallback
+            replacement = visualizer[replacement] or ctx_fallback
         else
-            replacement = fallback
+            replacement = ctx_fallback
         end
         return (C(pattern) * CargOne) / replacement
     end
@@ -500,7 +506,7 @@ local function visualize(content,settings) -- maybe also method in settings
             if trace_visualize then
                 report_visualizers("visualize using method %a",method)
             end
-            fallback(content,1,settings)
+            ctx_fallback(content,1,settings)
         end
     end
 end
@@ -568,7 +574,7 @@ local function realign(lines,strip) -- "yes", <number>
     if strip == v_yes then
         n = math.huge
         for i=1, #lines do
-            local spaces = find(lines[i],"%S")
+            local spaces = find(lines[i],"%S") -- can be lpeg
             if not spaces then
                 -- empty line
             elseif spaces == 0 then
@@ -592,11 +598,13 @@ local function realign(lines,strip) -- "yes", <number>
     return lines
 end
 
+local onlyspaces = S(" \t\f\n\r")^0 * P(-1)
+
 local function getstrip(lines,first,last)
     local first, last = first or 1, last or #lines
     for i=first,last do
         local li = lines[i]
-        if #li == 0 or find(li,"^%s*$") then
+        if #li == 0 or lpegmatch(onlyspaces,li) then
             first = first + 1
         else
             break
@@ -604,7 +612,7 @@ local function getstrip(lines,first,last)
     end
     for i=last,first,-1 do
         local li = lines[i]
-        if #li == 0 or find(li,"^%s*$") then
+        if #li == 0 or lpegmatch(onlyspaces,li) then
             last = last - 1
         else
             break
@@ -703,6 +711,7 @@ commands.loadvisualizer = visualizers.load
 function commands.typebuffer(settings)
     local lines = getlines(settings.name)
     if lines then
+        ctx_displayverbatiminitialize(#lines)
         local content, m = filter(lines,settings)
         if content and content ~= "" then
          -- content = decodecomment(content)
@@ -730,14 +739,21 @@ end
 -- parser so we use lpeg.
 --
 -- [[\text ]]  [[\text{}]]  [[\text \text ]]  [[\text \\ \text ]]
+--
+-- needed in e.g. tabulate (manuals)
 
------ strip = Cs((P(" ")^1 * P(-1)/"" + 1)^0)
-local strip = Cs((P("\\") * ((1-S("\\ "))^1) * (P(" ")/"") + 1)^0) --
+local compact_all  = Cs((P("\\") * ((1-S("\\ "))^1) * (P(" ")/"") * (P(-1) + S("[{")) + 1)^0)
+local compact_last = Cs((P(" ")^1 * P(-1)/"" + 1)^0)
 
 function commands.typestring(settings)
     local content = settings.data
     if content and content ~= "" then
-        content = #content > 1 and lpegmatch(strip,content) or content -- can be an option, but needed in e.g. tabulate
+        local compact = settings.compact
+        if compact == v_all then
+            content = lpegmatch(compact_all,content)
+        elseif compact == v_last then
+            content = lpegmatch(compact_last,content)
+        end
      -- content = decodecomment(content)
      -- content = dotabs(content,settings)
         visualize(content,checkedsettings(settings,"inline"))
