@@ -16,6 +16,9 @@ local lpegmatch, patterns = lpeg.match, lpeg.patterns
 local floor = math.floor
 
 -- extra functions, some might go (when not used)
+--
+-- we could serialize using %a but that won't work well is in the code we mostly use
+-- floats and as such we get unequality e.g. in version comparisons
 
 local stripper = patterns.stripper
 
@@ -85,6 +88,38 @@ local function sortedkeys(tab)
     end
 end
 
+local function sortedhashonly(tab)
+    if tab then
+        local srt, s = { }, 0
+        for key,_ in next, tab do
+            if type(key) == "string" then
+                s = s + 1
+                srt[s] = key
+            end
+        end
+        sort(srt)
+        return srt
+    else
+        return { }
+    end
+end
+
+local function sortedindexonly(tab)
+    if tab then
+        local srt, s = { }, 0
+        for key,_ in next, tab do
+            if type(key) == "number" then
+                s = s + 1
+                srt[s] = key
+            end
+        end
+        sort(srt)
+        return srt
+    else
+        return { }
+    end
+end
+
 local function sortedhashkeys(tab,cmp) -- fast one
     if tab then
         local srt, s = { }, 0
@@ -111,8 +146,10 @@ function table.allkeys(t)
     return sortedkeys(keys)
 end
 
-table.sortedkeys     = sortedkeys
-table.sortedhashkeys = sortedhashkeys
+table.sortedkeys      = sortedkeys
+table.sortedhashonly  = sortedhashonly
+table.sortedindexonly = sortedindexonly
+table.sortedhashkeys  = sortedhashkeys
 
 local function nothing() end
 
@@ -126,10 +163,13 @@ local function sortedhash(t,cmp)
             s = sortedkeys(t) -- the robust one
         end
         local n = 0
+        local m = #s
         local function kv(s)
-            n = n + 1
-            local k = s[n]
-            return k, t[k]
+            if n < m then
+                n = n + 1
+                local k = s[n]
+                return k, t[k]
+            end
         end
         return kv, s
     else
@@ -343,6 +383,7 @@ local noquotes, hexify, handle, reduce, compact, inline, functions
 local reserved = table.tohash { -- intercept a language inconvenience: no reserved words as key
     'and', 'break', 'do', 'else', 'elseif', 'end', 'false', 'for', 'function', 'if',
     'in', 'local', 'nil', 'not', 'or', 'repeat', 'return', 'then', 'true', 'until', 'while',
+    'NaN', 'goto',
 }
 
 local function simple_table(t)
@@ -363,12 +404,12 @@ local function simple_table(t)
                     else
                         tt[nt] = tostring(v) -- tostring not needed
                     end
-                elseif tv == "boolean" then
-                    nt = nt + 1
-                    tt[nt] = tostring(v)
                 elseif tv == "string" then
                     nt = nt + 1
                     tt[nt] = format("%q",v)
+                elseif tv == "boolean" then
+                    nt = nt + 1
+                    tt[nt] = v and "true" or "false"
                 else
                     tt = nil
                     break
@@ -394,7 +435,8 @@ end
 
 -- todo: %g faster on numbers than %s
 
--- we can speed this up with repeaters and formatters (is indeed faster)
+-- we can speed this up with repeaters and formatters but we haven't defined them
+-- yet
 
 local propername = patterns.propername -- was find(name,"^%a[%w%_]*$")
 
@@ -420,7 +462,7 @@ local function do_serialize(root,name,depth,level,indexed)
                     handle(format("%s[%q]={",depth,name))
                 end
             elseif tn == "boolean" then
-                handle(format("%s[%s]={",depth,tostring(name)))
+                handle(format("%s[%s]={",depth,name and "true" or "false"))
             else
                 handle(format("%s{",depth))
             end
@@ -456,21 +498,21 @@ local function do_serialize(root,name,depth,level,indexed)
             --~ if v == root then
                 -- circular
             --~ else
-            local t, tk = type(v), type(k)
+            local tv, tk = type(v), type(k)
             if compact and first and tk == "number" and k >= first and k <= last then
-                if t == "number" then
+                if tv == "number" then
                     if hexify then
                         handle(format("%s 0x%04X,",depth,v))
                     else
                         handle(format("%s %s,",depth,v)) -- %.99g
                     end
-                elseif t == "string" then
+                elseif tv == "string" then
                     if reduce and tonumber(v) then
                         handle(format("%s %s,",depth,v))
                     else
                         handle(format("%s %q,",depth,v))
                     end
-                elseif t == "table" then
+                elseif tv == "table" then
                     if not next(v) then
                         handle(format("%s {},",depth))
                     elseif inline then -- and #t > 0
@@ -483,11 +525,11 @@ local function do_serialize(root,name,depth,level,indexed)
                     else
                         do_serialize(v,k,depth,level+1,true)
                     end
-                elseif t == "boolean" then
-                    handle(format("%s %s,",depth,tostring(v)))
-                elseif t == "function" then
+                elseif tv == "boolean" then
+                    handle(format("%s %s,",depth,v and "true" or "false"))
+                elseif tv == "function" then
                     if functions then
-                        handle(format('%s load(%q),',depth,dump(v)))
+                        handle(format('%s load(%q),',depth,dump(v))) -- maybe strip
                     else
                         handle(format('%s "function",',depth))
                     end
@@ -498,7 +540,7 @@ local function do_serialize(root,name,depth,level,indexed)
                 if false then
                     handle(format("%s __p__=nil,",depth))
                 end
-            elseif t == "number" then
+            elseif tv == "number" then
                 if tk == "number" then
                     if hexify then
                         handle(format("%s [0x%04X]=0x%04X,",depth,k,v))
@@ -507,9 +549,9 @@ local function do_serialize(root,name,depth,level,indexed)
                     end
                 elseif tk == "boolean" then
                     if hexify then
-                        handle(format("%s [%s]=0x%04X,",depth,tostring(k),v))
+                        handle(format("%s [%s]=0x%04X,",depth,k and "true" or "false",v))
                     else
-                        handle(format("%s [%s]=%s,",depth,tostring(k),v)) -- %.99g
+                        handle(format("%s [%s]=%s,",depth,k and "true" or "false",v)) -- %.99g
                     end
                 elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
                     if hexify then
@@ -524,7 +566,7 @@ local function do_serialize(root,name,depth,level,indexed)
                         handle(format("%s [%q]=%s,",depth,k,v)) -- %.99g
                     end
                 end
-            elseif t == "string" then
+            elseif tv == "string" then
                 if reduce and tonumber(v) then
                     if tk == "number" then
                         if hexify then
@@ -533,7 +575,7 @@ local function do_serialize(root,name,depth,level,indexed)
                             handle(format("%s [%s]=%s,",depth,k,v))
                         end
                     elseif tk == "boolean" then
-                        handle(format("%s [%s]=%s,",depth,tostring(k),v))
+                        handle(format("%s [%s]=%s,",depth,k and "true" or "false",v))
                     elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
                         handle(format("%s %s=%s,",depth,k,v))
                     else
@@ -547,14 +589,14 @@ local function do_serialize(root,name,depth,level,indexed)
                             handle(format("%s [%s]=%q,",depth,k,v))
                         end
                     elseif tk == "boolean" then
-                        handle(format("%s [%s]=%q,",depth,tostring(k),v))
+                        handle(format("%s [%s]=%q,",depth,k and "true" or "false",v))
                     elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
                         handle(format("%s %s=%q,",depth,k,v))
                     else
                         handle(format("%s [%q]=%q,",depth,k,v))
                     end
                 end
-            elseif t == "table" then
+            elseif tv == "table" then
                 if not next(v) then
                     if tk == "number" then
                         if hexify then
@@ -563,7 +605,7 @@ local function do_serialize(root,name,depth,level,indexed)
                             handle(format("%s [%s]={},",depth,k))
                         end
                     elseif tk == "boolean" then
-                        handle(format("%s [%s]={},",depth,tostring(k)))
+                        handle(format("%s [%s]={},",depth,k and "true" or "false"))
                     elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
                         handle(format("%s %s={},",depth,k))
                     else
@@ -579,7 +621,7 @@ local function do_serialize(root,name,depth,level,indexed)
                                 handle(format("%s [%s]={ %s },",depth,k,concat(st,", ")))
                             end
                         elseif tk == "boolean" then
-                            handle(format("%s [%s]={ %s },",depth,tostring(k),concat(st,", ")))
+                            handle(format("%s [%s]={ %s },",depth,k and "true" or "false",concat(st,", ")))
                         elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
                             handle(format("%s %s={ %s },",depth,k,concat(st,", ")))
                         else
@@ -591,24 +633,24 @@ local function do_serialize(root,name,depth,level,indexed)
                 else
                     do_serialize(v,k,depth,level+1)
                 end
-            elseif t == "boolean" then
+            elseif tv == "boolean" then
                 if tk == "number" then
                     if hexify then
-                        handle(format("%s [0x%04X]=%s,",depth,k,tostring(v)))
+                        handle(format("%s [0x%04X]=%s,",depth,k,v and "true" or "false"))
                     else
-                        handle(format("%s [%s]=%s,",depth,k,tostring(v)))
+                        handle(format("%s [%s]=%s,",depth,k,v and "true" or "false"))
                     end
                 elseif tk == "boolean" then
-                    handle(format("%s [%s]=%s,",depth,tostring(k),tostring(v)))
+                    handle(format("%s [%s]=%s,",depth,tostring(k),v and "true" or "false"))
                 elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
-                    handle(format("%s %s=%s,",depth,k,tostring(v)))
+                    handle(format("%s %s=%s,",depth,k,v and "true" or "false"))
                 else
-                    handle(format("%s [%q]=%s,",depth,k,tostring(v)))
+                    handle(format("%s [%q]=%s,",depth,k,v and "true" or "false"))
                 end
-            elseif t == "function" then
+            elseif tv == "function" then
                 if functions then
-                    local f = getinfo(v).what == "C" and dump(dummy) or dump(v)
-                 -- local f = getinfo(v).what == "C" and dump(function(...) return v(...) end) or dump(v)
+                    local f = getinfo(v).what == "C" and dump(dummy) or dump(v) -- maybe strip
+                 -- local f = getinfo(v).what == "C" and dump(function(...) return v(...) end) or dump(v) -- maybe strip
                     if tk == "number" then
                         if hexify then
                             handle(format("%s [0x%04X]=load(%q),",depth,k,f))
@@ -616,7 +658,7 @@ local function do_serialize(root,name,depth,level,indexed)
                             handle(format("%s [%s]=load(%q),",depth,k,f))
                         end
                     elseif tk == "boolean" then
-                        handle(format("%s [%s]=load(%q),",depth,tostring(k),f))
+                        handle(format("%s [%s]=load(%q),",depth,k and "true" or "false",f))
                     elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
                         handle(format("%s %s=load(%q),",depth,k,f))
                     else
@@ -631,7 +673,7 @@ local function do_serialize(root,name,depth,level,indexed)
                         handle(format("%s [%s]=%q,",depth,k,tostring(v)))
                     end
                 elseif tk == "boolean" then
-                    handle(format("%s [%s]=%q,",depth,tostring(k),tostring(v)))
+                    handle(format("%s [%s]=%q,",depth,k and "true" or "false",tostring(v)))
                 elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
                     handle(format("%s %s=%q,",depth,k,tostring(v)))
                 else
@@ -713,321 +755,10 @@ local function serialize(_handle,root,name,specification) -- handle wins
     handle("}")
 end
 
---    -- This is some 20% faster than using format (because formatters are much faster) but
---    -- of course, inlining the format using .. is then again faster .. anyway, as we do
---    -- some pretty printing as well there is not that much to gain unless we make a 'fast'
---    -- ugly variant as well. But, we would have to move the formatter to l-string then.
-
---    local formatters = string.formatters
-
---    local function do_serialize(root,name,level,indexed)
---        if level > 0 then
---            if indexed then
---                handle(formatters["%w{"](level))
---            else
---                local tn = type(name)
---                if tn == "number" then
---                    if hexify then
---                        handle(formatters["%w[%04H]={"](level,name))
---                    else
---                        handle(formatters["%w[%s]={"](level,name))
---                    end
---                elseif tn == "string" then
---                    if noquotes and not reserved[name] and lpegmatch(propername,name) then
---                        handle(formatters["%w%s={"](level,name))
---                    else
---                        handle(formatters["%w[%q]={"](level,name))
---                    end
---                elseif tn == "boolean" then
---                    handle(formatters["%w[%S]={"](level,name))
---                else
---                    handle(formatters["%w{"](level))
---                end
---            end
---        end
---        -- we could check for k (index) being number (cardinal)
---        if root and next(root) then
---         -- local first, last = nil, 0 -- #root cannot be trusted here (will be ok in 5.2 when ipairs is gone)
---         -- if compact then
---         --     -- NOT: for k=1,#root do (we need to quit at nil)
---         --     for k,v in ipairs(root) do -- can we use next?
---         --         if not first then first = k end
---         --         last = last + 1
---         --     end
---         -- end
---            local first, last = nil, 0
---            if compact then
---                last = #root
---                for k=1,last do
---                    if root[k] == nil then
---                        last = k - 1
---                        break
---                    end
---                end
---                if last > 0 then
---                    first = 1
---                end
---            end
---            local sk = sortedkeys(root)
---            for i=1,#sk do
---                local k = sk[i]
---                local v = root[k]
---                --~ if v == root then
---                    -- circular
---                --~ else
---                local t, tk = type(v), type(k)
---                if compact and first and tk == "number" and k >= first and k <= last then
---                    if t == "number" then
---                        if hexify then
---                            handle(formatters["%w %04H,"](level,v))
---                        else
---                            handle(formatters["%w %s,"](level,v)) -- %.99g
---                        end
---                    elseif t == "string" then
---                        if reduce and tonumber(v) then
---                            handle(formatters["%w %s,"](level,v))
---                        else
---                            handle(formatters["%w %q,"](level,v))
---                        end
---                    elseif t == "table" then
---                        if not next(v) then
---                            handle(formatters["%w {},"](level))
---                        elseif inline then -- and #t > 0
---                            local st = simple_table(v)
---                            if st then
---                                handle(formatters["%w { %, t },"](level,st))
---                            else
---                                do_serialize(v,k,level+1,true)
---                            end
---                        else
---                            do_serialize(v,k,level+1,true)
---                        end
---                    elseif t == "boolean" then
---                        handle(formatters["%w %S,"](level,v))
---                    elseif t == "function" then
---                        if functions then
---                            handle(formatters['%w load(%q),'](level,dump(v)))
---                        else
---                            handle(formatters['%w "function",'](level))
---                        end
---                    else
---                        handle(formatters["%w %Q,"](level,v))
---                    end
---                elseif k == "__p__" then -- parent
---                    if false then
---                        handle(formatters["%w __p__=nil,"](level))
---                    end
---                elseif t == "number" then
---                    if tk == "number" then
---                        if hexify then
---                            handle(formatters["%w [%04H]=%04H,"](level,k,v))
---                        else
---                            handle(formatters["%w [%s]=%s,"](level,k,v)) -- %.99g
---                        end
---                    elseif tk == "boolean" then
---                        if hexify then
---                            handle(formatters["%w [%S]=%04H,"](level,k,v))
---                        else
---                            handle(formatters["%w [%S]=%s,"](level,k,v)) -- %.99g
---                        end
---                    elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
---                        if hexify then
---                            handle(formatters["%w %s=%04H,"](level,k,v))
---                        else
---                            handle(formatters["%w %s=%s,"](level,k,v)) -- %.99g
---                        end
---                    else
---                        if hexify then
---                            handle(formatters["%w [%q]=%04H,"](level,k,v))
---                        else
---                            handle(formatters["%w [%q]=%s,"](level,k,v)) -- %.99g
---                        end
---                    end
---                elseif t == "string" then
---                    if reduce and tonumber(v) then
---                        if tk == "number" then
---                            if hexify then
---                                handle(formatters["%w [%04H]=%s,"](level,k,v))
---                            else
---                                handle(formatters["%w [%s]=%s,"](level,k,v))
---                            end
---                        elseif tk == "boolean" then
---                            handle(formatters["%w [%S]=%s,"](level,k,v))
---                        elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
---                            handle(formatters["%w %s=%s,"](level,k,v))
---                        else
---                            handle(formatters["%w [%q]=%s,"](level,k,v))
---                        end
---                    else
---                        if tk == "number" then
---                            if hexify then
---                                handle(formatters["%w [%04H]=%q,"](level,k,v))
---                            else
---                                handle(formatters["%w [%s]=%q,"](level,k,v))
---                            end
---                        elseif tk == "boolean" then
---                            handle(formatters["%w [%S]=%q,"](level,k,v))
---                        elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
---                            handle(formatters["%w %s=%q,"](level,k,v))
---                        else
---                            handle(formatters["%w [%q]=%q,"](level,k,v))
---                        end
---                    end
---                elseif t == "table" then
---                    if not next(v) then
---                        if tk == "number" then
---                            if hexify then
---                                handle(formatters["%w [%04H]={},"](level,k))
---                            else
---                                handle(formatters["%w [%s]={},"](level,k))
---                            end
---                        elseif tk == "boolean" then
---                            handle(formatters["%w [%S]={},"](level,k))
---                        elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
---                            handle(formatters["%w %s={},"](level,k))
---                        else
---                            handle(formatters["%w [%q]={},"](level,k))
---                        end
---                    elseif inline then
---                        local st = simple_table(v)
---                        if st then
---                            if tk == "number" then
---                                if hexify then
---                                    handle(formatters["%w [%04H]={ %, t },"](level,k,st))
---                                else
---                                    handle(formatters["%w [%s]={ %, t },"](level,k,st))
---                                end
---                            elseif tk == "boolean" then
---                                handle(formatters["%w [%S]={ %, t },"](level,k,st))
---                            elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
---                                handle(formatters["%w %s={ %, t },"](level,k,st))
---                            else
---                                handle(formatters["%w [%q]={ %, t },"](level,k,st))
---                            end
---                        else
---                            do_serialize(v,k,level+1)
---                        end
---                    else
---                        do_serialize(v,k,level+1)
---                    end
---                elseif t == "boolean" then
---                    if tk == "number" then
---                        if hexify then
---                            handle(formatters["%w [%04H]=%S,"](level,k,v))
---                        else
---                            handle(formatters["%w [%s]=%S,"](level,k,v))
---                        end
---                    elseif tk == "boolean" then
---                        handle(formatters["%w [%S]=%S,"](level,k,v))
---                    elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
---                        handle(formatters["%w %s=%S,"](level,k,v))
---                    else
---                        handle(formatters["%w [%q]=%S,"](level,k,v))
---                    end
---                elseif t == "function" then
---                    if functions then
---                        local f = getinfo(v).what == "C" and dump(dummy) or dump(v)
---                     -- local f = getinfo(v).what == "C" and dump(function(...) return v(...) end) or dump(v)
---                        if tk == "number" then
---                            if hexify then
---                                handle(formatters["%w [%04H]=load(%q),"](level,k,f))
---                            else
---                                handle(formatters["%w [%s]=load(%q),"](level,k,f))
---                            end
---                        elseif tk == "boolean" then
---                            handle(formatters["%w [%S]=load(%q),"](level,k,f))
---                        elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
---                            handle(formatters["%w %s=load(%q),"](level,k,f))
---                        else
---                            handle(formatters["%w [%q]=load(%q),"](level,k,f))
---                        end
---                    end
---                else
---                    if tk == "number" then
---                        if hexify then
---                            handle(formatters["%w [%04H]=%Q,"](level,k,v))
---                        else
---                            handle(formatters["%w [%s]=%Q,"](level,k,v))
---                        end
---                    elseif tk == "boolean" then
---                        handle(formatters["%w [%S]=%Q,"](level,k,v))
---                    elseif noquotes and not reserved[k] and lpegmatch(propername,k) then
---                        handle(formatters["%w %s=%Q,"](level,k,v))
---                    else
---                        handle(formatters["%w [%q]=%Q,"](level,k,v))
---                    end
---                end
---                --~ end
---            end
---        end
---        if level > 0 then
---            handle(formatters["%w}"](level))
---        end
---    end
-
---    local function serialize(_handle,root,name,specification) -- handle wins
---        local tname = type(name)
---        if type(specification) == "table" then
---            noquotes  = specification.noquotes
---            hexify    = specification.hexify
---            handle    = _handle or specification.handle or print
---            reduce    = specification.reduce or false
---            functions = specification.functions
---            compact   = specification.compact
---            inline    = specification.inline and compact
---            if functions == nil then
---                functions = true
---            end
---            if compact == nil then
---                compact = true
---            end
---            if inline == nil then
---                inline = compact
---            end
---        else
---            noquotes  = false
---            hexify    = false
---            handle    = _handle or print
---            reduce    = false
---            compact   = true
---            inline    = true
---            functions = true
---        end
---        if tname == "string" then
---            if name == "return" then
---                handle("return {")
---            else
---                handle(name .. "={")
---            end
---        elseif tname == "number" then
---            if hexify then
---                handle(format("[0x%04X]={",name))
---            else
---                handle("[" .. name .. "]={")
---            end
---        elseif tname == "boolean" then
---            if name then
---                handle("return {")
---            else
---                handle("{")
---            end
---        else
---            handle("t={")
---        end
---        if root then
---            -- The dummy access will initialize a table that has a delayed initialization
---            -- using a metatable. (maybe explicitly test for metatable)
---            if getmetatable(root) then -- todo: make this an option, maybe even per subtable
---                local dummy = root._w_h_a_t_e_v_e_r_
---                root._w_h_a_t_e_v_e_r_ = nil
---            end
---            -- Let's forget about empty tables.
---            if next(root) then
---                do_serialize(root,name,0)
---            end
---        end
---        handle("}")
---    end
+-- A version with formatters is some 20% faster than using format (because formatters are
+-- much faster) but of course, inlining the format using .. is then again faster .. anyway,
+-- as we do some pretty printing as well there is not that much to gain unless we make a
+-- 'fast' ugly variant as well. But, we would have to move the formatter to l-string then.
 
 -- name:
 --
@@ -1309,7 +1040,9 @@ function table.print(t,...)
     end
 end
 
-setinspector(function(v) if type(v) == "table" then serialize(print,v,"table") return true end end)
+if setinspector then
+    setinspector(function(v) if type(v) == "table" then serialize(print,v,"table") return true end end)
+end
 
 -- -- -- obsolete but we keep them for a while and might comment them later -- -- --
 
@@ -1360,3 +1093,24 @@ function table.sorted(t,...)
     sort(t,...)
     return t -- still sorts in-place
 end
+
+--
+
+function table.values(t,s) -- optional sort flag
+    if t then
+        local values, keys, v = { }, { }, 0
+        for key, value in next, t do
+            if not keys[value] then
+                v = v + 1
+                values[v] = value
+                keys[k] = key
+            end
+        end
+        if s then
+            sort(values)
+        end
+        return values
+    else
+        return { }
+    end
+end
