@@ -1,3 +1,5 @@
+-- the tex table has overlap
+
 return {
  ["aleph"]={
   "AlephVersion",
@@ -76,13 +78,14 @@ return {
   "widowpenalties",
  },
  ["luatex"]={
+  "Uchar",
   "Udelcode",
   "Udelcodenum",
   "Udelimiter",
   "Udelimiterover",
   "Udelimiterunder",
   "Umathaccent",
-  "Umathaccents",
+--"Umathaccents",
   "Umathaxis",
   "Umathbinbinspacing",
   "Umathbinclosespacing",
@@ -92,7 +95,7 @@ return {
   "Umathbinordspacing",
   "Umathbinpunctspacing",
   "Umathbinrelspacing",
-  "Umathbotaccent",
+--"Umathbotaccent",
   "Umathchar",
   "Umathchardef",
   "Umathcharnum",
@@ -232,6 +235,7 @@ return {
   "luatexdatestamp",
   "luatexrevision",
   "luatexversion",
+  "luafunction",
   "mathstyle",
   "nokerns",
   "noligs",
@@ -410,7 +414,7 @@ return {
   "Udelimiterover",
   "Udelimiterunder",
   "Umathaccent",
-  "Umathaccents",
+--"Umathaccents",
   "Umathaxis",
   "Umathbinbinspacing",
   "Umathbinclosespacing",
@@ -420,7 +424,7 @@ return {
   "Umathbinordspacing",
   "Umathbinpunctspacing",
   "Umathbinrelspacing",
-  "Umathbotaccent",
+--"Umathbotaccent",
   "Umathchar",
   "Umathchardef",
   "Umathcharnum",
@@ -570,10 +574,10 @@ return {
   "catcodetable",
   "char",
   "chardef",
-  "chardp",
-  "charht",
-  "charit",
-  "charwd",
+--"chardp",
+--"charht",
+--"charit",
+--"charwd",
   "cleaders",
   "clearmarks",
   "closein",
@@ -1024,6 +1028,7 @@ return {
   "skewchar",
   "skip",
   "skipdef",
+--"skipexpr",
   "spacefactor",
   "spaceskip",
   "span",
@@ -1111,4 +1116,4 @@ return {
  ["xetex"]={
   "XeTeXversion",
  },
-}
\ No newline at end of file
+}
