@@ -7,51 +7,101 @@ if not modules then modules = { } end modules ['strc-reg'] = {
 }
 
 local next, type = next, type
-local texcount = tex.count
 local format, gmatch = string.format, string.gmatch
 local equal, concat, remove = table.are_equal, table.concat, table.remove
 local utfchar = utf.char
 local lpegmatch = lpeg.match
 local allocate = utilities.storage.allocate
 
-local trace_registers = false  trackers.register("structures.registers", function(v) trace_registers = v end)
+local trace_registers    = false  trackers.register("structures.registers", function(v) trace_registers = v end)
 
-local report_registers = logs.reporter("structure","registers")
+local report_registers     = logs.reporter("structure","registers")
 
-local structures      = structures
-local registers       = structures.registers
-local helpers         = structures.helpers
-local sections        = structures.sections
-local documents       = structures.documents
-local pages           = structures.pages
-local references      = structures.references
+local structures           = structures
+local registers            = structures.registers
+local helpers              = structures.helpers
+local sections             = structures.sections
+local documents            = structures.documents
+local pages                = structures.pages
+local references           = structures.references
 
-local mappings        = sorters.mappings
-local entries         = sorters.entries
-local replacements    = sorters.replacements
+local usedinternals        = references.usedinternals
 
-local processors      = typesetters.processors
-local splitprocessor  = processors.split
+local mappings             = sorters.mappings
+local entries              = sorters.entries
+local replacements         = sorters.replacements
 
-local variables       = interfaces.variables
-local context         = context
+local processors           = typesetters.processors
+local splitprocessor       = processors.split
 
-local matchingtilldepth, numberatdepth = sections.matchingtilldepth, sections.numberatdepth
+local texgetcount          = tex.getcount
+
+local variables            = interfaces.variables
+local v_forward            = variables.forward
+local v_all                = variables.all
+local v_yes                = variables.yes
+local v_current            = variables.current
+local v_previous           = variables.previous
+local v_text               = variables.text
+
+local context              = context
+local commands             = commands
+
+local matchingtilldepth    = sections.matchingtilldepth
+local numberatdepth        = sections.numberatdepth
+local currentlevel         = sections.currentlevel
+local currentid            = sections.currentid
+
+local touserdata           = helpers.touserdata
+
+local internalreferences   = references.internals
+local setinternalreference = references.setinternalreference
+
+local setmetatableindex    = table.setmetatableindex
+local texsetattribute      = tex.setattribute
+
+local a_destination        = attributes.private('destination')
+
+local absmaxlevel          = 5 -- \c_strc_registers_maxlevel
+
+local ctx_startregisteroutput   = context.startregisteroutput
+local ctx_stopregisteroutput    = context.stopregisteroutput
+local ctx_startregistersection  = context.startregistersection
+local ctx_stopregistersection   = context.stopregistersection
+local ctx_startregisterentries  = context.startregisterentries
+local ctx_stopregisterentries   = context.stopregisterentries
+local ctx_startregisterentry    = context.startregisterentry
+local ctx_stopregisterentry     = context.stopregisterentry
+local ctx_startregisterpages    = context.startregisterpages
+local ctx_stopregisterpages     = context.stopregisterpages
+local ctx_stopregisterseewords  = context.stopregisterseewords
+local ctx_startregisterseewords = context.startregisterseewords
+local ctx_registerentry         = context.registerentry
+local ctx_registerseeword       = context.registerseeword
+local ctx_registerpagerange     = context.registerpagerange
+local ctx_registeronepage       = context.registeronepage
 
 -- some day we will share registers and lists (although there are some conceptual
 -- differences in the application of keywords)
 
 local function filtercollected(names,criterium,number,collected,prevmode)
-    if not criterium or criterium == "" then criterium = variables.all end
-    local data = documents.data
-    local numbers, depth = data.numbers, data.depth
-    local hash, result, nofresult, all, detail = { }, { }, 0, not names or names == "" or names == variables.all, nil
+    if not criterium or criterium == "" then
+        criterium = v_all
+    end
+    local data      = documents.data
+    local numbers   = data.numbers
+    local depth     = data.depth
+    local hash      = { }
+    local result    = { }
+    local nofresult = 0
+    local all       = not names or names == "" or names == v_all
+    local detail    = nil
     if not all then
         for s in gmatch(names,"[^, ]+") do
             hash[s] = true
         end
     end
-    if criterium == variables.all or criterium == variables.text then
+    if criterium == v_all or criterium == v_text then
         for i=1,#collected do
             local v = collected[i]
             if all then
@@ -65,10 +115,11 @@ local function filtercollected(names,criterium,number,collected,prevmode)
                 end
             end
         end
-    elseif criterium == variables.current then
+    elseif criterium == v_current then
+        local collectedsections = sections.collected
         for i=1,#collected do
             local v = collected[i]
-            local sectionnumber = sections.collected[v.references.section]
+            local sectionnumber = collectedsections[v.references.section]
             if sectionnumber then
                 local cnumbers = sectionnumber.numbers
                 if prevmode then
@@ -103,10 +154,11 @@ local function filtercollected(names,criterium,number,collected,prevmode)
                 end
             end
         end
-    elseif criterium == variables.previous then
+    elseif criterium == v_previous then
+        local collectedsections = sections.collected
         for i=1,#collected do
             local v = collected[i]
-            local sectionnumber = sections.collected[v.references.section]
+            local sectionnumber = collectedsections[v.references.section]
             if sectionnumber then
                 local cnumbers = sectionnumber.numbers
                 if (all or hash[v.metadata.name]) and #cnumbers >= depth then
@@ -136,9 +188,9 @@ local function filtercollected(names,criterium,number,collected,prevmode)
         end
     elseif criterium == variables["local"] then
         if sections.autodepth(data.numbers) == 0 then
-            return filtercollected(names,variables.all,number,collected,prevmode)
+            return filtercollected(names,v_all,number,collected,prevmode)
         else
-            return filtercollected(names,variables.current,number,collected,prevmode)
+            return filtercollected(names,v_current,number,collected,prevmode)
         end
     else -- sectionname, number
         -- beware, this works ok for registers
@@ -188,44 +240,77 @@ registers.filtercollected = filtercollected
 -- result table; we might do that here as well but since sorting code is
 -- older we delay that decision
 
+-- maybe store the specification in the format (although we predefine only
+-- saved registers)
+
+local function checker(t,k)
+    local v = {
+        metadata = {
+            language = 'en',
+            sorted   = false,
+            class    = class,
+        },
+        entries  = { },
+    }
+    t[k] = v
+    return v
+end
+
 local function initializer()
     tobesaved = registers.tobesaved
     collected = registers.collected
-    local internals = references.internals
+    setmetatableindex(tobesaved,checker)
+    setmetatableindex(collected,checker)
+    local usedinternals = references.usedinternals
     for name, list in next, collected do
         local entries = list.entries
-        for e=1,#entries do
-            local entry = entries[e]
-            local r = entry.references
-            if r then
-                local internal = r and r.internal
-                if internal then
-                    internals[internal] = entry
+        if not list.metadata.notsaved then
+            for e=1,#entries do
+                local entry = entries[e]
+                local r = entry.references
+                if r then
+                    local internal = r and r.internal
+                    if internal then
+                        internalreferences[internal] = entry
+                        usedinternals[internal] = r.used
+                    end
                 end
             end
         end
     end
 end
 
-job.register('structures.registers.collected', tobesaved, initializer)
-
-local function allocate(class)
-    local d = tobesaved[class]
-    if not d then
-        d = {
-            metadata = {
-                language = 'en',
-                sorted   = false,
-                class    = class
-            },
-            entries  = { },
-        }
-        tobesaved[class] = d
+local function finalizer()
+    local flaginternals = references.flaginternals
+    for k, v in next, tobesaved do
+        local entries = v.entries
+        if entries then
+            for i=1,#entries do
+                local r = entries[i].references
+                if r and flaginternals[r.internal] then
+                    r.used = true
+                end
+            end
+        end
     end
-    return d
 end
 
-registers.define = allocate
+job.register('structures.registers.collected', tobesaved, initializer, finalizer)
+
+setmetatableindex(tobesaved,checker)
+setmetatableindex(collected,checker)
+
+local function defineregister(class,method)
+    local d = tobesaved[class]
+    if method == v_forward then
+        d.metadata.notsaved = true
+    end
+end
+
+registers.define           = defineregister -- 4 times is somewhat over the top but we want consistency
+registers.setmethod        = defineregister -- and we might have a difference some day
+commands.defineregister    = defineregister
+commands.setregistermethod = defineregister
 
 local entrysplitter = lpeg.tsplitat('+') -- & obsolete in mkiv
 
@@ -234,7 +319,6 @@ local tagged = { }
 local function preprocessentries(rawdata)
     local entries = rawdata.entries
     if entries then
---~ table.print(rawdata)
         local e, k = entries[1] or "", entries[2] or ""
         local et, kt, entryproc, pageproc
         if type(e) == "table" then
@@ -250,14 +334,15 @@ local function preprocessentries(rawdata)
             kt = lpegmatch(entrysplitter,k)
         end
         entries = { }
-        for k=1,#et do
-            entries[k] = { et[k] or "", kt[k] or "" }
-        end
+        local ok = false
         for k=#et,1,-1 do
-            if entries[k][1] ~= "" then
-                break
-            else
+            local etk = et[k]
+            local ktk = kt[k]
+            if not ok and etk == "" then
                 entries[k] = nil
+            else
+                entries[k] = { etk or "", ktk ~= "" and ktk or nil }
+                ok = true
             end
         end
         rawdata.list = entries
@@ -272,44 +357,94 @@ local function preprocessentries(rawdata)
     end
 end
 
-function registers.store(rawdata) -- metadata, references, entries
-    local data = allocate(rawdata.metadata.name).entries
+local function storeregister(rawdata) -- metadata, references, entries
     local references = rawdata.references
-    references.realpage = references.realpage or 0 -- just to be sure as it can be refered to
+    local metadata   = rawdata.metadata
+    -- checking
+    if not metadata.kind then
+        metadata.kind = "entry"
+    end
+    --
+    if not metadata.catcodes then
+        metadata.catcodes = tex.catcodetable -- get
+    end
+    --
+    local name     = metadata.name
+    local notsaved = tobesaved[name].metadata.notsaved
+    --
+    local internal = references.internal
+    if not internal then
+        internal = texgetcount("locationcount") -- we assume that it has been set
+        references.internal = internal
+    end
+    --
+    if notsaved then
+        usedinternals[internal] = true -- todo view (we assume that forward references index entries are used)
+    end
+    --
+    if not references.realpage then
+        references.realpage = 0 -- just to be sure as it can be refered to
+    end
+    --
+    local userdata = rawdata.userdata
+    if userdata then
+        rawdata.userdata = touserdata(userdata)
+    end
+    --
+    references.section = currentid()
+    metadata.level     = currentlevel()
+    --
+    local data     = notsaved and collected[name] or tobesaved[name]
+    local entries  = data.entries
+    internalreferences[internal] = rawdata
     preprocessentries(rawdata)
-    data[#data+1] = rawdata
+    entries[#entries+1] = rawdata
     local label = references.label
-    if label and label ~= "" then tagged[label] = #data end
-    context(#data)
+    if label and label ~= "" then
+        tagged[label] = #entries
+    else
+        references.label = nil
+    end
+    return #entries
 end
 
-function registers.enhance(name,n)
-    local r = tobesaved[name].entries[n]
-    if r then
-        r.references.realpage = texcount.realpageno
+local function enhanceregister(name,n)
+    local data = tobesaved[name].metadata.notsaved and collected[name] or tobesaved[name]
+    local entry = data.entries[n]
+    if entry then
+        entry.references.realpage = texgetcount("realpageno")
     end
 end
 
-function registers.extend(name,tag,rawdata) -- maybe do lastsection internally
+local function extendregister(name,tag,rawdata) -- maybe do lastsection internally
     if type(tag) == "string" then
         tag = tagged[tag]
     end
     if tag then
-        local r = tobesaved[name].entries[tag]
-        if r then
-            local rr = r.references
-            rr.lastrealpage = texcount.realpageno
-            rr.lastsection = sections.currentid()
+        local data = tobesaved[name].metadata.notsaved and collected[name] or tobesaved[name]
+        local entry = data.entries[tag]
+        if entry then
+            local references = entry.references
+            references.lastrealpage = texgetcount("realpageno")
+            references.lastsection = currentid()
             if rawdata then
+                local userdata = rawdata.userdata
+                if userdata then
+                    rawdata.userdata = touserdata(userdata)
+                end
                 if rawdata.entries then
                     preprocessentries(rawdata)
                 end
-                for k,v in next, rawdata do
-                    if not r[k] then
-                        r[k] = v
+                local metadata = rawdata.metadata
+                if metadata and not metadata.catcodes then
+                    metadata.catcodes = tex.catcodetable -- get
+                end
+                for k, v in next, rawdata do
+                    local rk = references[k]
+                    if not rk then
+                        references[k] = v
                     else
-                        local rk = r[k]
-                        for kk,vv in next, v do
+                        for kk, vv in next, v do
                             if type(vv) == "table" then
                                 if next(vv) then
                                     rk[kk] = vv
@@ -325,6 +460,19 @@ function registers.extend(name,tag,rawdata) -- maybe do lastsection internally
     end
 end
 
+registers.store   = storeregister
+registers.enhance = enhanceregister
+registers.extend  = extendregister
+
+function commands.storeregister(rawdata)
+    local nofentries = storeregister(rawdata)
+    setinternalreference(nil,nil,rawdata.references.internal)
+    context(nofentries)
+end
+
+commands.enhanceregister = enhanceregister
+commands.extendregister  = extendregister
+
 -- sorting and rendering
 
 local compare = sorters.comparers.basic
@@ -334,7 +482,8 @@ function registers.compare(a,b)
     if result ~= 0 then
         return result
     else
-        local ka, kb = a.metadata.kind, b.metadata.kind
+        local ka = a.metadata.kind
+        local kb = b.metadata.kind
         if ka == kb then
             local page_a, page_b = a.references.realpage, b.references.realpage
             if not page_a or not page_b then
@@ -448,17 +597,19 @@ end
 
 function registers.prepare(data)
     -- data has 'list' table
-    local strip = sorters.strip
+    local strip    = sorters.strip
     local splitter = sorters.splitters.utf
-    local result = data.result
+    local result   = data.result
     if result then
         for i=1, #result do
-            local entry, split = result[i], { }
-            local list = entry.list
+            local entry = result[i]
+            local split = { }
+            local list  = entry.list
             if list then
                 for l=1,#list do
-                    local ll = list[l]
-                    local word, key = ll[1], ll[2]
+                    local ll   = list[l]
+                    local word = ll[1]
+                    local key  = ll[2]
                     if not key or key == "" then
                         key = word
                     end
@@ -473,7 +624,11 @@ function registers.prepare(data)
 end
 
 function registers.sort(data,options)
-    sorters.sort(data.result,registers.compare)
+ -- if options.pagenumber == false then
+ --     sorters.sort(data.result,compare)
+ -- else
+        sorters.sort(data.result,registers.compare)
+ -- end
 end
 
 function registers.unique(data,options)
@@ -482,7 +637,8 @@ function registers.unique(data,options)
     for k=1,#dataresult do
         local v = dataresult[k]
         if prev then
-            local pr, vr = prev.references, v.references
+            local vr = v.references
+            local pr = prev.references
             if not equal(prev.list,v.list) then
                 -- ok
             elseif pr.realpage ~= vr.realpage then
@@ -525,10 +681,11 @@ function registers.finalize(data,options) -- maps character to index (order)
             if trace_registers then
                 report_registers("splitting at %a",tag)
             end
-            done, nofdone = { }, 0
+            done     = { }
+            nofdone  = 0
             nofsplit = nofsplit + 1
+            lasttag  = tag
             split[nofsplit] = { tag = tag, data = done }
-            lasttag = tag
         end
         nofdone = nofdone + 1
         done[nofdone] = v
@@ -536,7 +693,7 @@ function registers.finalize(data,options) -- maps character to index (order)
     data.result = split
 end
 
-function registers.analyzed(class,options)
+local function analyzeregister(class,options)
     local data = collected[class]
     if data and data.entries then
         options = options or { }
@@ -553,9 +710,21 @@ function registers.analyzed(class,options)
     end
 end
 
+registers.analyze = analyzeregister
+
+function registers.analyze(class,options)
+    context(analyzeregister(class,options))
+end
+
+
 -- todo take conversion from index
 
 function registers.userdata(index,name)
+    local data = references.internals[tonumber(index)]
+    return data and data.userdata and data.userdata[name] or nil
+end
+
+function commands.registeruserdata(index,name)
     local data = references.internals[tonumber(index)]
     data = data and data.userdata and data.userdata[name]
     if data then
@@ -565,22 +734,26 @@ end
 
 -- todo: ownnumber
 
+local h_prefixpage     = helpers.prefixpage
+local h_prefixlastpage = helpers.prefixlastpage
+local h_title          = helpers.title
+
 local function pagerange(f_entry,t_entry,is_last,prefixspec,pagespec)
     local fer, ter = f_entry.references, t_entry.references
-    context.registerpagerange(
+    ctx_registerpagerange(
         f_entry.processors and f_entry.processors[2] or "",
         fer.internal or 0,
         fer.realpage or 0,
         function()
-            helpers.prefixpage(f_entry,prefixspec,pagespec)
+            h_prefixpage(f_entry,prefixspec,pagespec)
         end,
         ter.internal or 0,
         ter.lastrealpage or ter.realpage or 0,
         function()
             if is_last then
-                helpers.prefixlastpage(t_entry,prefixspec,pagespec) -- swaps page and realpage keys
+                h_prefixlastpage(t_entry,prefixspec,pagespec) -- swaps page and realpage keys
             else
-                helpers.prefixpage    (t_entry,prefixspec,pagespec)
+                h_prefixpage    (t_entry,prefixspec,pagespec)
             end
         end
     )
@@ -588,11 +761,11 @@ end
 
 local function pagenumber(entry,prefixspec,pagespec)
     local er = entry.references
-    context.registeronepage(
+    ctx_registeronepage(
         entry.processors and entry.processors[2] or "",
         er.internal or 0,
         er.realpage or 0,
-        function() helpers.prefixpage(entry,prefixspec,pagespec) end
+        function() h_prefixpage(entry,prefixspec,pagespec) end
     )
 end
 
@@ -660,17 +833,39 @@ local function collapsepages(pages)
 end
 
 function registers.flush(data,options,prefixspec,pagespec)
-    local collapse_singles = options.compress == variables.yes
-    local collapse_ranges  = options.compress == variables.all
+    local collapse_singles = options.compress == v_yes
+    local collapse_ranges  = options.compress == v_all
+    local show_page_number = options.pagenumber ~= false -- true or false
     local result = data.result
-    context.startregisteroutput()
+    local maxlevel = 0
+    --
+    for i=1,#result do
+        local data = result[i].data
+        for d=1,#data do
+            local m = #data[d].list
+            if m > maxlevel then
+                maxlevel = m
+            end
+        end
+    end
+    if maxlevel > absmaxlevel then
+        maxlevel = absmaxlevel
+        report_registers("limiting level to %a",maxlevel)
+    end
+    --
+    ctx_startregisteroutput()
+    local done    = { }
+    local started = false
     for i=1,#result do
      -- ranges need checking !
         local sublist = result[i]
-        local done = { false, false, false, false }
+     -- local done = { false, false, false, false }
+        for i=1,maxlevel do
+            done[i] = false
+        end
         local data = sublist.data
         local d, n = 0, 0
-        context.startregistersection(sublist.tag)
+        ctx_startregistersection(sublist.tag)
         for d=1,#data do
             local entry = data[d]
             if entry.metadata.kind == "see" then
@@ -683,131 +878,154 @@ function registers.flush(data,options,prefixspec,pagespec)
                 end
             end
         end
+        -- ok, this is tricky: we use e[i] delayed so we need it to be local
+        -- but we don't want to allocate too many entries so there we go
         while d < #data do
             d = d + 1
             local entry = data[d]
-            local e = { false, false, false, false }
+            local e = { false, false, false }
+            for i=3,maxlevel do
+                e[i] = false
+            end
             local metadata = entry.metadata
             local kind = metadata.kind
             local list = entry.list
-            for i=1,4 do -- max 4
+            for i=1,maxlevel do
                 if list[i] then
                     e[i] = list[i][1]
                 end
                 if e[i] ~= done[i] then
                     if e[i] and e[i] ~= "" then
                         done[i] = e[i]
+                        for j=i+1,maxlevel do
+                            done[j] = false
+                        end
+                        if started then
+                            ctx_stopregisterentry()
+                            started = false
+                        end
                         if n == i then
-                            context.stopregisterentries()
-                            context.startregisterentries(n)
+--                             ctx_stopregisterentries()
+--                             ctx_startregisterentries(n)
                         else
                             while n > i do
                                 n = n - 1
-                                context.stopregisterentries()
+                                ctx_stopregisterentries()
                             end
                             while n < i do
                                 n = n + 1
-                                context.startregisterentries(n)
+                                ctx_startregisterentries(n)
                             end
                         end
-                        local internal  = entry.references.internal or 0
-                        local seeparent = entry.references.seeparent or ""
-                        local processor = entry.processors and entry.processors[1] or ""
+                        local references = entry.references
+                        local processors = entry.processors
+                        local internal  = references.internal or 0
+                        local seeparent = references.seeparent or ""
+                        local processor = processors and processors[1] or ""
+                        -- so, we need to keep e as is (local), or we need local title = e[i] ... which might be
+                        -- more of a problem
+                        ctx_startregisterentry(0) -- will become a counter
+                        started = true
                         if metadata then
-                            context.registerentry(processor,internal,seeparent,function() helpers.title(e[i],metadata) end)
+                            ctx_registerentry(processor,internal,seeparent,function() h_title(e[i],metadata) end)
                         else -- ?
-                            context.registerentry(processor,internal,seeindex,e[i])
+                            ctx_registerentry(processor,internal,seeindex,e[i])
                         end
                     else
                         done[i] = false
+                        for j=i+1,maxlevel do
+                            done[j] = false
+                        end
                     end
                 end
             end
             if kind == 'entry' then
-                context.startregisterpages()
-                if collapse_singles or collapse_ranges then
-                    -- we collapse ranges and keep existing ranges as they are
-                    -- so we get prebuilt as well as built ranges
-                    local first, last, prev, pages, dd, nofpages = entry, nil, entry, { }, d, 0
-                    while dd < #data do
-                        dd = dd + 1
-                        local next = data[dd]
-                        if next and next.metadata.kind == "see" then
-                            dd = dd - 1
-                            break
-                        else
-                            local el, nl = entry.list, next.list
-                            if not equal(el,nl) then
+                if show_page_number then
+                    ctx_startregisterpages()
+                    if collapse_singles or collapse_ranges then
+                        -- we collapse ranges and keep existing ranges as they are
+                        -- so we get prebuilt as well as built ranges
+                        local first, last, prev, pages, dd, nofpages = entry, nil, entry, { }, d, 0
+                        while dd < #data do
+                            dd = dd + 1
+                            local next = data[dd]
+                            if next and next.metadata.kind == "see" then
                                 dd = dd - 1
-                            --~ first = nil
                                 break
-                            elseif next.references.lastrealpage then
-                                nofpages = nofpages + 1
-                                pages[nofpages] = first and { first, last or first } or { entry, entry }
-                                nofpages = nofpages + 1
-                                pages[nofpages] = { next, next }
-                                first, last, prev = nil, nil, nil
-                            elseif not first then
-                                first, prev = next, next
-                            elseif next.references.realpage - prev.references.realpage == 1 then -- 1 ?
-                                last, prev = next, next
                             else
-                                nofpages = nofpages + 1
-                                pages[nofpages] = { first, last or first }
-                                first, last, prev = next, nil, next
-                            end
-                        end
-                    end
-                    if first then
-                        nofpages = nofpages + 1
-                        pages[nofpages] = { first, last or first }
-                    end
-                    if collapse_ranges and nofpages > 1 then
-                        nofpages = collapsepages(pages)
-                    end
-                    if nofpages > 0 then -- or 0
-                        d = dd
-                        for p=1,nofpages do
-                            local first, last = pages[p][1], pages[p][2]
-                            if first == last then
-                                if first.references.lastrealpage then
-                                    pagerange(first,first,true,prefixspec,pagespec)
+                                local el, nl = entry.list, next.list
+                                if not equal(el,nl) then
+                                    dd = dd - 1
+                                --~ first = nil
+                                    break
+                                elseif next.references.lastrealpage then
+                                    nofpages = nofpages + 1
+                                    pages[nofpages] = first and { first, last or first } or { entry, entry }
+                                    nofpages = nofpages + 1
+                                    pages[nofpages] = { next, next }
+                                    first, last, prev = nil, nil, nil
+                                elseif not first then
+                                    first, prev = next, next
+                                elseif next.references.realpage - prev.references.realpage == 1 then -- 1 ?
+                                    last, prev = next, next
                                 else
-                                    pagenumber(first,prefixspec,pagespec)
+                                    nofpages = nofpages + 1
+                                    pages[nofpages] = { first, last or first }
+                                    first, last, prev = next, nil, next
                                 end
-                            elseif last.references.lastrealpage then
-                                pagerange(first,last,true,prefixspec,pagespec)
-                            else
-                                pagerange(first,last,false,prefixspec,pagespec)
                             end
                         end
-                    elseif entry.references.lastrealpage then
-                        pagerange(entry,entry,true,prefixspec,pagespec)
-                    else
-                        pagenumber(entry,prefixspec,pagespec)
-                    end
-                else
-                    while true do
-                        if entry.references.lastrealpage then
+                        if first then
+                            nofpages = nofpages + 1
+                            pages[nofpages] = { first, last or first }
+                        end
+                        if collapse_ranges and nofpages > 1 then
+                            nofpages = collapsepages(pages)
+                        end
+                        if nofpages > 0 then -- or 0
+                            d = dd
+                            for p=1,nofpages do
+                                local first, last = pages[p][1], pages[p][2]
+                                if first == last then
+                                    if first.references.lastrealpage then
+                                        pagerange(first,first,true,prefixspec,pagespec)
+                                    else
+                                        pagenumber(first,prefixspec,pagespec)
+                                    end
+                                elseif last.references.lastrealpage then
+                                    pagerange(first,last,true,prefixspec,pagespec)
+                                else
+                                    pagerange(first,last,false,prefixspec,pagespec)
+                                end
+                            end
+                        elseif entry.references.lastrealpage then
                             pagerange(entry,entry,true,prefixspec,pagespec)
                         else
                             pagenumber(entry,prefixspec,pagespec)
                         end
-                        if d == #data then
-                            break
-                        else
-                            d = d + 1
-                            local next = data[d]
-                            if next.metadata.kind == "see" or not equal(entry.list,next.list) then
-                                d = d - 1
+                    else
+                        while true do
+                            if entry.references.lastrealpage then
+                                pagerange(entry,entry,true,prefixspec,pagespec)
+                            else
+                                pagenumber(entry,prefixspec,pagespec)
+                            end
+                            if d == #data then
                                 break
                             else
-                                entry = next
+                                d = d + 1
+                                local next = data[d]
+                                if next.metadata.kind == "see" or not equal(entry.list,next.list) then
+                                    d = d - 1
+                                    break
+                                else
+                                    entry = next
+                                end
                             end
                         end
                     end
+                    ctx_stopregisterpages()
                 end
-                context.stopregisterpages()
             elseif kind == 'see' then
                 local t, nt = { }, 0
                 while true do
@@ -826,37 +1044,46 @@ function registers.flush(data,options,prefixspec,pagespec)
                         end
                     end
                 end
-                context.startregisterseewords()
+                ctx_startregisterseewords()
                 for i=1,nt do
                     local entry = t[i]
                     local seeword   = entry.seeword
                     local seetext   = seeword.text or ""
                     local processor = seeword.processor or (entry.processors and entry.processors[1]) or ""
                     local seeindex  = entry.references.seeindex or ""
-                    context.registerseeword(i,n,processor,0,seeindex,seetext)
+                    ctx_registerseeword(i,n,processor,0,seeindex,seetext)
                 end
-                context.stopregisterseewords()
+                ctx_stopregisterseewords()
             end
         end
+        if started then
+            ctx_stopregisterentry()
+            started = false
+        end
         while n > 0 do
-            context.stopregisterentries()
+            ctx_stopregisterentries()
             n = n - 1
         end
-        context.stopregistersection()
+        ctx_stopregistersection()
     end
-    context.stopregisteroutput()
+    ctx_stopregisteroutput()
     -- for now, maybe at some point we will do a multipass or so
     data.result = nil
     data.metadata.sorted = false
+    -- temp hack for luajittex :
+    local entries = data.entries
+    for i=1,#entries do
+        entries[i].split = nil
+    end
+ -- collectgarbage("collect")
 end
 
-function registers.analyze(class,options)
-    context(registers.analyzed(class,options))
-end
-
-function registers.process(class,...)
-    if registers.analyzed(class,...) > 0 then
-        registers.flush(collected[class],...)
+local function processregister(class,...)
+    if analyzeregister(class,...) > 0 then
+        local data = collected[class]
+        registers.flush(data,...)
     end
 end
 
+registers.process        = processregister
+commands.processregister = processregister
