@@ -35,7 +35,9 @@ end
 
 local function failure_one(name)
     name_two = gsub(name,"%-.*$","")
-    if name_two == name then
+    if name == "loc" then
+        -- ignore
+    elseif name_two == name then
         report_typescripts("unknown library %a",name_one)
     else
         commands.uselibrary {
